@@ -17,7 +17,6 @@ import pytest_asyncio
 import redis.asyncio as redis
 from redis.exceptions import ConnectionError
 from redis.typing import EncodableT
-from redis.utils import HIREDIS_AVAILABLE
 from tests.conftest import get_protocol_version, skip_if_server_version_lt
 
 from .compat import aclosing, create_task, mock
@@ -461,10 +460,9 @@ class TestPubSubMessages:
 
 @pytest.mark.onlynoncluster
 class TestPubSubRESP3Handler:
-    def my_handler(self, message):
+    async def my_handler(self, message):
         self.message = ["my handler", message]
 
-    @pytest.mark.skipif(HIREDIS_AVAILABLE, reason="PythonParser only")
     async def test_push_handler(self, r):
         if get_protocol_version(r) in [2, "2", None]:
             return
@@ -1050,9 +1048,11 @@ class TestBaseException:
         assert msg is not None
         # timeout waiting for another message which never arrives
         assert pubsub.connection.is_connected
-        with patch("redis._parsers._AsyncRESP2Parser.read_response") as mock1, patch(
-            "redis._parsers._AsyncHiredisParser.read_response"
-        ) as mock2, patch("redis._parsers._AsyncRESP3Parser.read_response") as mock3:
+        with (
+            patch("redis._parsers._AsyncRESP2Parser.read_response") as mock1,
+            patch("redis._parsers._AsyncHiredisParser.read_response") as mock2,
+            patch("redis._parsers._AsyncRESP3Parser.read_response") as mock3,
+        ):
             mock1.side_effect = BaseException("boom")
             mock2.side_effect = BaseException("boom")
             mock3.side_effect = BaseException("boom")
