@@ -1,14 +1,14 @@
 import bz2
 import csv
 import os
-import time
+import asyncio
 from io import TextIOWrapper
 
 import numpy as np
 import pytest
 import pytest_asyncio
+from redis import ResponseError
 import redis.asyncio as redis
-import redis.commands.search
 import redis.commands.search.aggregation as aggregations
 import redis.commands.search.reducers as reducers
 from redis.commands.search import AsyncSearch
@@ -19,7 +19,7 @@ from redis.commands.search.field import (
     TextField,
     VectorField,
 )
-from redis.commands.search.indexDefinition import IndexDefinition, IndexType
+from redis.commands.search.index_definition import IndexDefinition, IndexType
 from redis.commands.search.query import GeoFilter, NumericFilter, Query
 from redis.commands.search.result import Result
 from redis.commands.search.suggestion import Suggestion
@@ -27,6 +27,8 @@ from tests.conftest import (
     is_resp2_connection,
     skip_if_redis_enterprise,
     skip_if_resp_version,
+    skip_if_server_version_gte,
+    skip_if_server_version_lt,
     skip_ifmodversion_lt,
 )
 
@@ -47,8 +49,8 @@ async def decoded_r(create_redis, stack_url):
 async def waitForIndex(env, idx, timeout=None):
     delay = 0.1
     while True:
-        res = await env.execute_command("FT.INFO", idx)
         try:
+            res = await env.execute_command("FT.INFO", idx)
             if int(res[res.index("indexing") + 1]) == 0:
                 break
         except ValueError:
@@ -59,8 +61,12 @@ async def waitForIndex(env, idx, timeout=None):
                     break
             except ValueError:
                 break
+        except ResponseError:
+            # index doesn't exist yet
+            # continue to sleep and try again
+            pass
 
-        time.sleep(delay)
+        await asyncio.sleep(delay)
         if timeout is not None:
             timeout -= delay
             if timeout <= 0:
@@ -339,6 +345,7 @@ async def test_client(decoded_r: redis.Redis):
 
 @pytest.mark.redismod
 @pytest.mark.onlynoncluster
+@skip_if_server_version_gte("7.9.0")
 async def test_scores(decoded_r: redis.Redis):
     await decoded_r.ft().create_index((TextField("txt"),))
 
@@ -359,6 +366,29 @@ async def test_scores(decoded_r: redis.Redis):
         assert "doc1" == res["results"][1]["id"]
 
 
+@pytest.mark.redismod
+@pytest.mark.onlynoncluster
+@skip_if_server_version_lt("7.9.0")
+async def test_scores_with_new_default_scorer(decoded_r: redis.Redis):
+    await decoded_r.ft().create_index((TextField("txt"),))
+
+    await decoded_r.hset("doc1", mapping={"txt": "foo baz"})
+    await decoded_r.hset("doc2", mapping={"txt": "foo bar"})
+
+    q = Query("foo ~bar").with_scores()
+    res = await decoded_r.ft().search(q)
+    if is_resp2_connection(decoded_r):
+        assert 2 == res.total
+        assert "doc2" == res.docs[0].id
+        assert 0.87 == pytest.approx(res.docs[0].score, 0.01)
+        assert "doc1" == res.docs[1].id
+    else:
+        assert 2 == res["total_results"]
+        assert "doc2" == res["results"][0]["id"]
+        assert 0.87 == pytest.approx(res["results"][0]["score"], 0.01)
+        assert "doc1" == res["results"][1]["id"]
+
+
 @pytest.mark.redismod
 async def test_stopwords(decoded_r: redis.Redis):
     stopwords = ["foo", "bar", "baz"]
@@ -661,7 +691,7 @@ async def test_summarize(decoded_r: redis.Redis):
     await createIndex(decoded_r.ft())
     await waitForIndex(decoded_r, "idx")
 
-    q = Query("king henry").paging(0, 1)
+    q = Query('"king henry"').paging(0, 1)
     q.highlight(fields=("play", "txt"), tags=("<b>", "</b>"))
     q.summarize("txt")
 
@@ -673,7 +703,7 @@ async def test_summarize(decoded_r: redis.Redis):
             == doc.txt
         )
 
-        q = Query("king henry").paging(0, 1).summarize().highlight()
+        q = Query('"king henry"').paging(0, 1).summarize().highlight()
 
         doc = sorted((await decoded_r.ft().search(q)).docs)[0]
         assert "<b>Henry</b> ... " == doc.play
@@ -689,7 +719,7 @@ async def test_summarize(decoded_r: redis.Redis):
             == doc["extra_attributes"]["txt"]
         )
 
-        q = Query("king henry").paging(0, 1).summarize().highlight()
+        q = Query('"king henry"').paging(0, 1).summarize().highlight()
 
         doc = sorted((await decoded_r.ft().search(q))["results"])[0]
         assert "<b>Henry</b> ... " == doc["extra_attributes"]["play"]
@@ -1025,6 +1055,9 @@ async def test_phonetic_matcher(decoded_r: redis.Redis):
 
 @pytest.mark.redismod
 @pytest.mark.onlynoncluster
+# NOTE(imalinovskyi): This test contains hardcoded scores valid only for RediSearch 2.8+
+@skip_ifmodversion_lt("2.8.0", "search")
+@skip_if_server_version_gte("7.9.0")
 async def test_scorer(decoded_r: redis.Redis):
     await decoded_r.ft().create_index((TextField("description"),))
 
@@ -1083,6 +1116,69 @@ async def test_scorer(decoded_r: redis.Redis):
         assert 0.0 == res["results"][0]["score"]
 
 
+@pytest.mark.redismod
+@pytest.mark.onlynoncluster
+# NOTE(imalinovskyi): This test contains hardcoded scores valid only for RediSearch 2.8+
+@skip_ifmodversion_lt("2.8.0", "search")
+@skip_if_server_version_lt("7.9.0")
+async def test_scorer_with_new_default_scorer(decoded_r: redis.Redis):
+    await decoded_r.ft().create_index((TextField("description"),))
+
+    await decoded_r.hset(
+        "doc1", mapping={"description": "The quick brown fox jumps over the lazy dog"}
+    )
+    await decoded_r.hset(
+        "doc2",
+        mapping={
+            "description": "Quick alice was beginning to get very tired of sitting by her quick sister on the bank, and of having nothing to do."  # noqa
+        },
+    )
+
+    if is_resp2_connection(decoded_r):
+        # default scorer is BM25STD
+        res = await decoded_r.ft().search(Query("quick").with_scores())
+        assert 0.23 == pytest.approx(res.docs[0].score, 0.05)
+        res = await decoded_r.ft().search(Query("quick").scorer("TFIDF").with_scores())
+        assert 1.0 == res.docs[0].score
+        res = await decoded_r.ft().search(
+            Query("quick").scorer("TFIDF.DOCNORM").with_scores()
+        )
+        assert 0.14285714285714285 == res.docs[0].score
+        res = await decoded_r.ft().search(Query("quick").scorer("BM25").with_scores())
+        assert 0.22471909420069797 == res.docs[0].score
+        res = await decoded_r.ft().search(Query("quick").scorer("DISMAX").with_scores())
+        assert 2.0 == res.docs[0].score
+        res = await decoded_r.ft().search(
+            Query("quick").scorer("DOCSCORE").with_scores()
+        )
+        assert 1.0 == res.docs[0].score
+        res = await decoded_r.ft().search(
+            Query("quick").scorer("HAMMING").with_scores()
+        )
+        assert 0.0 == res.docs[0].score
+    else:
+        res = await decoded_r.ft().search(Query("quick").with_scores())
+        assert 0.23 == pytest.approx(res["results"][0]["score"], 0.05)
+        res = await decoded_r.ft().search(Query("quick").scorer("TFIDF").with_scores())
+        assert 1.0 == res["results"][0]["score"]
+        res = await decoded_r.ft().search(
+            Query("quick").scorer("TFIDF.DOCNORM").with_scores()
+        )
+        assert 0.14285714285714285 == res["results"][0]["score"]
+        res = await decoded_r.ft().search(Query("quick").scorer("BM25").with_scores())
+        assert 0.22471909420069797 == res["results"][0]["score"]
+        res = await decoded_r.ft().search(Query("quick").scorer("DISMAX").with_scores())
+        assert 2.0 == res["results"][0]["score"]
+        res = await decoded_r.ft().search(
+            Query("quick").scorer("DOCSCORE").with_scores()
+        )
+        assert 1.0 == res["results"][0]["score"]
+        res = await decoded_r.ft().search(
+            Query("quick").scorer("HAMMING").with_scores()
+        )
+        assert 0.0 == res["results"][0]["score"]
+
+
 @pytest.mark.redismod
 async def test_get(decoded_r: redis.Redis):
     await decoded_r.ft().create_index((TextField("f1"), TextField("f2")))
@@ -1109,6 +1205,7 @@ async def test_get(decoded_r: redis.Redis):
 @pytest.mark.redismod
 @pytest.mark.onlynoncluster
 @skip_ifmodversion_lt("2.2.0", "search")
+@skip_if_server_version_gte("7.9.0")
 async def test_config(decoded_r: redis.Redis):
     assert await decoded_r.ft().config_set("TIMEOUT", "100")
     with pytest.raises(redis.ResponseError):
@@ -1119,6 +1216,19 @@ async def test_config(decoded_r: redis.Redis):
     assert "100" == res["TIMEOUT"]
 
 
+@pytest.mark.redismod
+@pytest.mark.onlynoncluster
+@skip_if_server_version_lt("7.9.0")
+async def test_config_with_removed_ftconfig(decoded_r: redis.Redis):
+    assert await decoded_r.config_set("timeout", "100")
+    with pytest.raises(redis.ResponseError):
+        await decoded_r.config_set("timeout", "null")
+    res = await decoded_r.config_get("*")
+    assert "100" == res["timeout"]
+    res = await decoded_r.config_get("timeout")
+    assert "100" == res["timeout"]
+
+
 @pytest.mark.redismod
 @pytest.mark.onlynoncluster
 async def test_aggregations_groupby(decoded_r: redis.Redis):
@@ -1497,14 +1607,14 @@ async def test_withsuffixtrie(decoded_r: redis.Redis):
     if is_resp2_connection(decoded_r):
         info = await decoded_r.ft().info()
         assert "WITHSUFFIXTRIE" not in info["attributes"][0]
-        assert await decoded_r.ft().dropindex("idx")
+        assert await decoded_r.ft().dropindex()
 
         # create withsuffixtrie index (text field)
         assert await decoded_r.ft().create_index(TextField("t", withsuffixtrie=True))
         await waitForIndex(decoded_r, getattr(decoded_r.ft(), "index_name", "idx"))
         info = await decoded_r.ft().info()
         assert "WITHSUFFIXTRIE" in info["attributes"][0]
-        assert await decoded_r.ft().dropindex("idx")
+        assert await decoded_r.ft().dropindex()
 
         # create withsuffixtrie index (tag field)
         assert await decoded_r.ft().create_index(TagField("t", withsuffixtrie=True))
@@ -1514,14 +1624,14 @@ async def test_withsuffixtrie(decoded_r: redis.Redis):
     else:
         info = await decoded_r.ft().info()
         assert "WITHSUFFIXTRIE" not in info["attributes"][0]["flags"]
-        assert await decoded_r.ft().dropindex("idx")
+        assert await decoded_r.ft().dropindex()
 
         # create withsuffixtrie index (text fields)
         assert await decoded_r.ft().create_index(TextField("t", withsuffixtrie=True))
         await waitForIndex(decoded_r, getattr(decoded_r.ft(), "index_name", "idx"))
         info = await decoded_r.ft().info()
         assert "WITHSUFFIXTRIE" in info["attributes"][0]["flags"]
-        assert await decoded_r.ft().dropindex("idx")
+        assert await decoded_r.ft().dropindex()
 
         # create withsuffixtrie index (tag field)
         assert await decoded_r.ft().create_index(TagField("t", withsuffixtrie=True))
@@ -1644,7 +1754,7 @@ async def test_search_commands_in_pipeline(decoded_r: redis.Redis):
 @pytest.mark.redismod
 async def test_query_timeout(decoded_r: redis.Redis):
     q1 = Query("foo").timeout(5000)
-    assert q1.get_args() == ["foo", "TIMEOUT", 5000, "LIMIT", 0, 10]
+    assert q1.get_args() == ["foo", "TIMEOUT", 5000, "DIALECT", 2, "LIMIT", 0, 10]
     q2 = Query("foo").timeout("not_a_number")
     with pytest.raises(redis.ResponseError):
         await decoded_r.ft().search(q2)
@@ -1659,7 +1769,7 @@ async def test_binary_and_text_fields(decoded_r: redis.Redis):
     mixed_data = {"first_name": "🐍python", "vector_emb": fake_vec.tobytes()}
     await decoded_r.hset(f"{index_name}:1", mapping=mixed_data)
 
-    schema = (
+    schema = [
         TagField("first_name"),
         VectorField(
             "embeddings_bio",
@@ -1670,7 +1780,7 @@ async def test_binary_and_text_fields(decoded_r: redis.Redis):
                 "DISTANCE_METRIC": "COSINE",
             },
         ),
-    )
+    ]
 
     await decoded_r.ft(index_name).create_index(
         fields=schema,
@@ -1678,6 +1788,7 @@ async def test_binary_and_text_fields(decoded_r: redis.Redis):
             prefix=[f"{index_name}:"], index_type=IndexType.HASH
         ),
     )
+    await waitForIndex(decoded_r, index_name)
 
     query = (
         Query("*")
@@ -1687,14 +1798,20 @@ async def test_binary_and_text_fields(decoded_r: redis.Redis):
     result = await decoded_r.ft(index_name).search(query=query, query_params={})
     docs = result.docs
 
+    if len(docs) == 0:
+        hash_content = await decoded_r.hget(f"{index_name}:1", "first_name")
+    assert len(docs) > 0, (
+        f"Returned search results are empty. Result: {result}; Hash: {hash_content}"
+    )
+
     decoded_vec_from_search_results = np.frombuffer(
         docs[0]["vector_emb"], dtype=np.float32
     )
 
-    assert np.array_equal(
-        decoded_vec_from_search_results, fake_vec
-    ), "The vectors are not equal"
+    assert np.array_equal(decoded_vec_from_search_results, fake_vec), (
+        "The vectors are not equal"
+    )
 
-    assert (
-        docs[0]["first_name"] == mixed_data["first_name"]
-    ), "The text field is not decoded correctly"
+    assert docs[0]["first_name"] == mixed_data["first_name"], (
+        "The text field is not decoded correctly"
+    )
